@@ -174,38 +174,39 @@ static void handle_hsp_key_press(void *buf, uint16_t len)
  * index in table equals to 'opcode - HAL_MINIMUM_EVENT' */
 static const struct hal_ipc_handler ev_handlers[] = {
 	/* HAL_EV_HANDSFREE_CONN_STATE */
-	{handle_conn_state, false, sizeof(struct hal_ev_handsfree_conn_state)},
+	{ handle_conn_state, false,
+				sizeof(struct hal_ev_handsfree_conn_state) },
 	/* HAL_EV_HANDSFREE_AUDIO_STATE */
-	{handle_audio_state, false,
-				sizeof(struct hal_ev_handsfree_audio_state)},
+	{ handle_audio_state, false,
+				sizeof(struct hal_ev_handsfree_audio_state) },
 	/* HAL_EV_HANDSFREE_VR */
-	{handle_vr_state, false, sizeof(struct hal_ev_handsfree_vr_state)},
+	{ handle_vr_state, false, sizeof(struct hal_ev_handsfree_vr_state) },
 	/*HAL_EV_HANDSFREE_ANSWER */
-	{handle_answer, false, 0},
+	{ handle_answer, false, 0 },
 	/*HAL_EV_HANDSFREE_HANGUP */
-	{handle_hangup, false, 0},
+	{ handle_hangup, false, 0 },
 	/* HAL_EV_HANDSFREE_VOLUME */
-	{handle_volume, false, sizeof(struct hal_ev_handsfree_volume)},
+	{ handle_volume, false, sizeof(struct hal_ev_handsfree_volume) },
 	/* HAL_EV_HANDSFREE_DIAL */
-	{handle_dial, true, sizeof(struct hal_ev_handsfree_dial)},
+	{ handle_dial, true, sizeof(struct hal_ev_handsfree_dial) },
 	/* HAL_EV_HANDSFREE_DTMF */
-	{handle_dtmf, false, sizeof(struct hal_ev_handsfree_dtmf)},
+	{ handle_dtmf, false, sizeof(struct hal_ev_handsfree_dtmf) },
 	/* HAL_EV_HANDSFREE_NREC */
-	{handle_nrec, false, sizeof(struct hal_ev_handsfree_nrec)},
+	{ handle_nrec, false, sizeof(struct hal_ev_handsfree_nrec) },
 	/* HAL_EV_HANDSFREE_CHLD */
-	{handle_chld, false, sizeof(struct hal_ev_handsfree_chld)},
+	{ handle_chld, false, sizeof(struct hal_ev_handsfree_chld) },
 	/* HAL_EV_HANDSFREE_CNUM */
-	{handle_cnum, false, 0},
+	{ handle_cnum, false, 0 },
 	/* HAL_EV_HANDSFREE_CIND */
-	{handle_cind, false, 0},
+	{ handle_cind, false, 0 },
 	/* HAL_EV_HANDSFREE_COPS */
-	{handle_cops, false, 0},
+	{ handle_cops, false, 0 },
 	/* HAL_EV_HANDSFREE_CLCC */
-	{handle_clcc, false, 0},
+	{ handle_clcc, false, 0 },
 	/* HAL_EV_HANDSFREE_UNKNOWN_AT */
-	{handle_unknown_at, true, sizeof(struct hal_ev_handsfree_unknown_at)},
+	{ handle_unknown_at, true, sizeof(struct hal_ev_handsfree_unknown_at) },
 	/* HAL_EV_HANDSFREE_HSP_KEY_PRESS */
-	{handle_hsp_key_press, false, 0},
+	{ handle_hsp_key_press, false, 0 },
 };
 
 static uint8_t get_mode(void)
