@@ -27,15 +27,15 @@
 struct input_device;
 struct input_conn;
 
-int input_device_register(struct btd_device *device, const char *path,
-				const char *uuid, const sdp_record_t *rec,
-				int timeout);
-int input_device_unregister(const char *path, const char *uuid);
+void input_set_idle_timeout(int timeout);
 
+int input_device_register(struct btd_service *service);
+void input_device_unregister(struct btd_service *service);
+
+bool input_device_exists(const bdaddr_t *src, const bdaddr_t *dst);
 int input_device_set_channel(const bdaddr_t *src, const bdaddr_t *dst, int psm,
 							GIOChannel *io);
 int input_device_close_channels(const bdaddr_t *src, const bdaddr_t *dst);
 
-int input_device_connect(struct btd_device *dev, struct btd_profile *profile);
-int input_device_disconnect(struct btd_device *dev,
-						struct btd_profile *profile);
+int input_device_connect(struct btd_service *service);
+int input_device_disconnect(struct btd_service *service);
